@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright © 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-bookmarks-export.h,v 1.2 2004/03/17 09:08:08 marco Exp $
+ *  $Id$
  */
 
 #ifndef EPHY_BOOKMARKS_EXPORT_H
@@ -28,6 +28,9 @@ G_BEGIN_DECLS
 void ephy_bookmarks_export_rdf (EphyBookmarks *bookmarks,
 				const char *filename);
 
+void ephy_bookmarks_export_mozilla (EphyBookmarks *bookmarks,
+				    const char *filename);
+
 G_END_DECLS
 
 #endif
